import base64
import hashlib
import random
import os
from time import sleep
from psutil import disk_partitions
from bilibili_api import video
import requests
import json
import asyncio


# 向开发者和用户问好
def welcome():
    print('你好开发者~ 欢迎使用HanLib，如需查看详细信息请调用HanLib.about()')
    print('你好用户~ 欢迎使用本程序！这个程序使用了HanLib，如果您学过python也可以试试哦！')


# 生成临时UID
# 可选:
#   uid: 回翻译的uid
# 如果 传入uid:
# 那么 返回加密uid
# 否则 返回随机加密uid
def encode_uid(uid=None):
    pie = '14159265358979'  # 圆周率派
    a = random.randint(0, len(pie) - 1)  # 圆周率后第N位的下标
    if uid is None:
        b = random.randint(1000000000, 9999999999)  # 真临时UID
    else:
        b = uid  # 真临时UID
    c = int(pie[a]) * 1111111111  # 圆周率后第N位 × 1111111111(十个一) -> 生成一个掩码
    c = int(c) + b  # 掩码 + 真临时UID
    uid = str(c) + str(a) + str(b) + 'H' + str(len(str(c))) + 'H' + str(a) + 'H' + str(b)  # 全部连接在一起
    return uid, b


# 回翻译临时UID
# 传入: 加密的uid
def decode_uid(uid):
    pie = '14159265358979'  # 圆周率派
    uid_list = uid.split('H')  # 分割
    temp = uid_list[0]  # 获取临时UID
    temp2 = int(uid_list[1])  # 获取参数
    true_uid = temp[0:temp2]  # 去除无意义字符
    true_uid = int(true_uid) - 1111111111 * int(pie[int(uid_list[-2])])  # 去除掩码
    return true_uid


# 什么是HanLib
def about():
    print('HanLib是HanTools开发者闲着无聊的时候写的\n'
          '我把HanTools里所有可能用到的函数都摆在着了\n'
          '你可以试试调用HanLib.encode_uid(一串int类型的数据，最长十位)，当然，也可以不传参，不传则随即生成真uid\n'
          '然后存进变量里\n'
          '在使用HanLib.decode_uid(变量名)，之后也存进变量\n'
          '分别输出出来\n'
          '然后你就得到了随机生成的临时uid（加密的）\n'
          '和加密之前的uid')


# sha256加密
# 传入: 未加密的文本
def en_sha256(text):
    t = hashlib.sha256()
    t.update(text.encode(encoding='utf-8'))
    return t.hexdigest()


# base64加密
# 传入: 未加密的文本
# 注意: 会反复加密随机次，如不需要请删除
def en_base64(text):
    # 反复加密 ran 次
    ran = random.randint(1, 10)
    for i in range(ran):
        text = base64.b64encode(text.encode('utf-8'))
        text = str(text, encoding='utf-8')
    return text, str(ran)


# base64解密
def de_base64(text):
    text = text.split('H')
    ran = text[-1]
    t = text[4]
    for i in range(ran):
        t = base64.b64decode(t.encode('utf-8'))
        t = str(t, encoding='utf-8')
    return t


# 验证收件人是不是用户
# 传入
def verification(uid, other_uid, text):
    text = text.split('HH')
    # 验证收件人
    if text[1] == uid:
        code = True
        # 验证发件人
        if text[0] == other_uid:
            code = True
            # 验证文本是否被篡改
            if en_sha256(text[-2]) == text[2]:
                code = True
            else:
                code = False
                return code
        else:
            code = False
            return code
    else:
        code = False
        return code
    return code


# 对文本进行编码
def encode(uid, other_uid, text):
    t, r = en_base64(text)
    encode_text = '{0}HH{1}HH{2}HH225fh7e2HH{3}HH{4}'.format(str(uid), str(other_uid), str(en_sha256(t)), str(t),
                                                             str(r))
    return encode_text


# 对文本进行编码解码
def decode(uid, other_uid, text):
    if verification(uid, other_uid, text):
        text = text.split('HH')
        t = text[-2]
        for i in range(int(text[-1])):
            t = base64.b64decode(t.encode('utf-8'))
            t = str(t, encoding='utf-8')
        return t
    else:
        print('HanTools尊重每位用户的隐私，经程序校验，这条不是发给你的，或者被篡改，HanTools强制终止了此程序的运行')
        exit()


# 生成日志
# 必须包含“log[Debug].txt”文件
# 必须包含“log[xxx].txt”
# level = “log[xxx].txt”中的“xxx”
# msg = 日志信息
# encoding = 文件编码（默认GBK）
# output = 0/1 || 0 = 不在控制台输出 || 1 = 在控制台输出 || 默认1
def log(level, msg, encoding='GBK', output=1):
    if output:
        print('[%s] %s' % (level, msg))
    with open('log[%s].txt' % level, 'a', encoding=encoding) as f:
        f.write('[%s] %s\n' % (level, msg))
    with open('../../pythonProject6/log[Debug].txt', 'a', encoding=encoding) as f:
        f.write('[%s] %s\n' % (level, msg))


# 查找可移动设备（若没有会等待一秒，继续查找）
def find_driver():
    while True:
        #  设置休眠时间
        sleep(1)
        #  检测所有的驱动器，进行遍历寻找
        for item in disk_partitions():
            if 'removable' in item.opts:
                driver, opts = item.device, item.opts
                return driver


# 提取文件的sha256
def CalcFileSha256(filename):
    with open(filename, "rb") as f:
        sha256obj = hashlib.sha256()
        sha256obj.update(f.read())
        hash_value: str = sha256obj.hexdigest()
        return hash_value


# 列出文件夹下所有的目录与文件
def get_all_path(open_file_path):
    root_dir = open_file_path
    path_list = []
    list = os.listdir(root_dir)  # 列出文件夹下所有的目录与文件
    for i in range(0, len(list)):
        com_path = os.path.join(root_dir, list[i])
        if os.path.isfile(com_path):
            path_list.append(com_path)
        if os.path.isdir(com_path):
            path_list.extend(get_all_path(com_path))
    return path_list


# 将bilibili某位up主的近期0-100个视频的播放量转为折线图
# 输入:
#   uid: up主的uid
#   ps:  视频数量 || 范围: 0-100
# 返回:
#   x: x轴的数据 (播放量)
#   y: y轴的数据 (第n个视频 || 从旧到新)
def bili_look(uid, ps):
    y = []
    x = []
    if ps > 100:
        raise ValueError('视频数量 必须小于等于100，否则返回的JSON状态码是-400')
    home_url = "https://api.bilibili.com/x/space/arc/search?mid=%d&ps=%d&tid=0&pn=1&keyword=&order=pubdate&jsonp=jsonp" \
               % (uid, ps)

    headers = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:81.0) Gecko/20100101 Firefox/81.0'
    }
    html = requests.get(url=home_url, headers=headers)
    html_content = html.text
    json_html = json.loads(html_content)
    print('[INFO] 正在获取JSON：')
    print(json_html)
    print('[INFO] 正在获取BV号：')
    for i in json_html['data']['list']['vlist']:
        print(i['bvid'])
    for i in range(len(json_html['data']['list']['vlist']) + 1, 1, -1):
        x.append(i)

    async def main(bvid):
        # 实例化 Video 类
        v = video.Video(bvid=bvid)
        # 获取信息
        info = await v.get_info()
        # 打印信息
        y.append(info['stat']['view'])
        print(bvid, '=', info['stat']['view'])

    for i in json_html['data']['list']['vlist']:
        print('[INFO] 正在获取播放量')
        asyncio.get_event_loop().run_until_complete(main(i['bvid']))
    return x, y
